
/* cruelty :) */

/* Copyright (C) 1997 Ian Main
 * Copyright (C) 2001 ksr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gtk/gtk.h>

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>

#include "list.h"
#include "interface.h"
#include "dillo.h"
#include "nav.h"
#include "browser.h"
#include "menu.h"
#include "misc.h"
#include "url.h"
#include "debug.h"

#define LOAD_ENCODINGS 1
#define SAVE_ENCODING 2
#define CLOSE_ENCODINGS 3

/* this #define will cut page title if > 39 chars */
#define TITLE39

/* double quote */
#define D_QUOTE 0x22

/* Data types */

typedef struct _Encoding Encoding;
typedef struct _CallbackInfo CallbackInfo;

struct _Encoding {
   char *title;
   char *iconv_name;
   GtkWidget *menuitem;
};

struct _CallbackInfo {
   BrowserWindow *bw;
   guint index;
};

/*
 * Forward declarations
 */
static void
   Charset_load_to_menu(),
   Charset_file_op(gint operation, const char *title, const char *iconv_name),
   Charset_save_to_file(FILE *fp, const char *title,  const char *iconv_name);

static char*
   Charset_search_line(char *line, char *start_text, char *end_text);

/*
 * Local data
 */
static Encoding *encodings = NULL;
static gint num_encodings = 0;
static gint num_encodings_max = 16;

char *encodings_menu_data = 0;
static char *_encodings_menu_data = 
"<enc value=\"ASCII\">ASCII</enc>\n"
"<enc value=\"JISAUTO\">Japanese (Autodetect)</enc>\n"
"<enc value=\"SJIS\">Japanese (ShiftJIS)</enc>\n"
"<enc value=\"EUCJP\">Japanese (EUCJP)</enc>\n"
"<enc value=\"ISO-2022-JP\">Japanese (ISO-2022-JP)</enc>\n"
"<enc value=\"ISO8859-15\">Western (ISO8859-15)</enc>\n"
"<enc value=\"ISO8859-2\">Central European (ISO8859-2)</enc>\n"
"<enc value=\"CP1250\">Central European (Windows 1250)</enc>\n"
"<enc value=\"UTF-8\">Unicode (UTF-8)</enc>\n"
"<enc value=\"KOI8-R\">Cyrillic (KOI8-R)</enc>\n"
"<enc value=\"CP1251\">Cyrillic (CP1251)</enc>\n"
"<enc value=\"IBM866\">Cyrillic (IBM866)</enc>\n"
"<enc value=\"BIG5\">Traditional Chinese (Big5)</enc>\n"
"<enc value=\"zh_TW-euc\">Traditional Chinese (EUC-TW)</enc>\n"
"<enc value=\"gb2312\">Simplified Chinese (GB2312)</enc>\n"
"<enc value=\"ISO8859-7\">Greek (ISO8859-7)</enc>\n"
"<enc value=\"CP1253\">Greek (Windows 1253)</enc>\n"
"<enc value=\"ISO8859-9\">Turkish (ISO8859-9)</enc>\n"
"<enc value=\"EUC-KR\">Korean (EUCKR)</enc>\n"
"<enc value=\"ISO-2022-KR\">Korean (ISO2022 KR)</enc>\n";


static struct charsets {
  char *name;
  int code;
} charsets[] = {
  {"JISAUTO", DILLO_CHARSET_GUESS},
  {"SJIS", DILLO_CHARSET_SJIS},
  {"EUCJP", DILLO_CHARSET_EUCJP},
  {"ISO-2022-JP", DILLO_CHARSET_ISO2022JP},
  {"BIG5", DILLO_CHARSET_BIG5},
  {"EUC-KR",DILLO_CHARSET_EUCKR},
  {"ISO-2022-KR", DILLO_CHARSET_ISO2022KR},
};
#define NTBL (sizeof(charsets)/sizeof(charsets[0]))

int a_Charset_get_charset_code(char *name)
{
  int i = 0;
  for (i = 0; i < NTBL; i++) {
    if (!strcmp(charsets[i].name, name)) return charsets[i].code;
  }
  return DILLO_CHARSET_USERDEFINED;
}

/*
 * Allocate memory and load the encodings list
 */
void a_Charset_init(void)
{
   gchar *file;

   /* Here we load and set the encodings */
   file = a_Misc_prepend_user_home(".dillo/encodings");
   Charset_file_op(LOAD_ENCODINGS, file, NULL);
   g_free(file);
}

/*
 * ?
 */
void Charset_set_encoding(GtkWidget *widget, CallbackInfo *CbInfo)
{
   if (CbInfo->index >= num_encodings) {
      g_warning("encoding not found!\n");
      return;
   }
   g_print("Selected encoding: %s\n",encodings[CbInfo->index].iconv_name);
   CbInfo->bw->encoding = encodings[CbInfo->index].iconv_name;
   a_Nav_reload(CbInfo->bw, 0);
   /*HERE!!! LOOKOUT!!!! */
}

/*
 * Add a encoding to the encodings menu of a particular browser window
 */
void Charset_add_to_menu(BrowserWindow *bw, GtkWidget *menuitem, guint index)
{
   CallbackInfo *CbInfo;

   gtk_menu_append(GTK_MENU(bw->enc_menu), menuitem);

   CbInfo = g_new(CallbackInfo, 1);
   CbInfo->bw = bw;
   CbInfo->index = index;

   /* accelerator goes here */
   gtk_signal_connect(GTK_OBJECT (menuitem), "activate",
                      (GtkSignalFunc)Charset_set_encoding, CbInfo);
}

/*
 * ?
 */
static GtkWidget *Charset_insert(const char *title, const char *iconv_name)
{
   GtkWidget *menuitem;

   menuitem = gtk_menu_item_new_with_label(title);
   gtk_widget_show(menuitem);

   a_List_add(encodings, num_encodings, sizeof(*encodings), num_encodings_max);
   encodings[num_encodings].title = g_strdup(title);
   encodings[num_encodings].iconv_name = g_strdup(iconv_name);
   encodings[num_encodings].menuitem = menuitem;
   num_encodings++;
   return menuitem;
}

/*
 * Add the new encoding to encodings menu of _all_ browser windows and then
 * write the new encoding to file
 */
/*
void a_Charset_add(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *)client_data;
   gint i;
#ifdef TITLE39
   gboolean allocated = FALSE;
#endif
   char *title;
   char *iconv_name;
   GtkWidget *menuitem;

   title = bw->menu_popup.info.title;
   iconv_name = bw->menu_popup.info.iconv_name;

#ifdef TITLE39
   if (strlen (title) > 39) {
      char buf1[20];
      char buf2[20];

      memcpy (buf1, title, 18);
      buf1[18] = '\0';
      strcpy (buf2, title + strlen (title) - 18);
      buf2[18] = '\0';
      title = g_strconcat (buf1, "...", buf2, NULL);
      allocated = TRUE;
   }
#endif

   menuitem = Charset_insert(title, iconv_name);
   Charset_add_to_menu(browser_window[0], menuitem, num_encodings-1);
   for (i = 1; i < num_bw; i++) {
      menuitem= gtk_menu_item_new_with_label(encodings[num_encodings-1].title);
      gtk_widget_show(menuitem);
      Charset_add_to_menu(browser_window[i], menuitem, num_encodings-1);
   }

      Charset_file_op(SAVE_ENCODING, title, iconv_name);

#ifdef TITLE39
   if (allocated)
      g_free (title);
#endif
}
*/
/*
 * Never called (the file remains open all the time)  --Jcid
 */
void Charset_close(void)
{
   Charset_file_op(CLOSE_ENCODINGS, NULL, NULL);
   g_free(encodings_menu_data);
}

/*
 * Performs operations on the encoding file..
 * for first call, title is the filename
 */
static void
 Charset_file_op(gint operation, const char *title, const char *iconv_name)
{
   FILE *fp;
   static gint initialized = 0;

   if (!initialized) {
      if (operation == LOAD_ENCODINGS) {
	if ((fp = fopen(title, "r")) == NULL) {
	  g_print("dillo: opening encoding file %s: %s\n",
		  title, strerror(errno));
	recover:
	  encodings_menu_data = strcpy(g_malloc(strlen(_encodings_menu_data)+1),_encodings_menu_data); 
	   initialized = 1;	  
         } else {
	   struct stat stat;
	   int len;
	   initialized = 1;
	   if (fstat(fileno(fp), &stat) == -1) goto recover;
	   len = stat.st_size;
	   encodings_menu_data = g_malloc(len+1);
	   if (len != fread(encodings_menu_data, 1, len, fp)) goto recover;
	   encodings_menu_data[len] = 0;
	   fclose(fp); fp = 0;
	 }
      } else {
         g_print("Error: invalid call to Charset_file_op.\n");
      }
   }

   g_return_if_fail( initialized );

   switch (operation) {
   case LOAD_ENCODINGS:
      Charset_load_to_menu();
      break;

      /*   case SAVE_ENCODING:
      Charset_save_to_file(fp, title, iconv_name);
      break;

   case CLOSE_ENCODINGS:
      fclose(fp);
      break;*/

   default:
      break;
   }
}

/*
 * Save encodings to ~/.dillo/encodings
 */
static void
 Charset_save_to_file(FILE *fp, const char *title, const char* iconv_name)
{
   fseek(fp, 0L, SEEK_END);
   fprintf(fp, "<enc value=\"%s\">%s</a>\n", iconv_name,title);
   fflush(fp);
}

/*
 * Load encodings
 */
static void Charset_load_to_menu()
{
   gchar *title=NULL;
   gchar *iconv_name=NULL;
   char buf[4096], *s = buf;
   gint i = 0;
   GtkWidget *menuitem;
   char *p = encodings_menu_data;

   g_print("Loading encodings...\n");
   while (1) {
      /* Read a whole line from the file */
     char *r = buf;
     if (*p == 0) break;
     while (*p != '\n' && *p != 0 && r - s < sizeof(buf) ) *r++ = *p++;
     if (*p == '\n') p++;
     *r = 0;

      /* get url from line */
      if ( !(iconv_name = Charset_search_line(buf, "=\"", "\">")) )
         continue;

      /* get title from line */
      if ( !(title = Charset_search_line(buf, "\">", "</")) ){
         g_free(iconv_name);
         continue;
      }
      /*      printf("%s = %s\n",title,iconv_name);*/
      menuitem = Charset_insert(title, iconv_name);
      Charset_add_to_menu(browser_window[0], menuitem, i);
      g_free(iconv_name);
      g_free(title);
      i++;
   }
}

/*
 * Copy encodings when new browser windows are opened.
 * Called by 'a_Menu_mainbar_new()'
 */
void a_Charset_fill_new_menu(BrowserWindow *bw)
{
   gint i;
   GtkWidget *menuitem;

   for (i = 0; i < num_encodings; i++) {
     menuitem = gtk_menu_item_new_with_label(encodings[i].title);
     gtk_widget_show(menuitem);
     Charset_add_to_menu(bw, menuitem, i);
   }
}

static char *
 Charset_search_line(char *line, char *start_text, char *end_text) {
   gint segment_length;
   char *start_index, *end_index;

   /* if string is not found, return NULL */
   if ((start_index = strstr(line, start_text)) == NULL)
      return (NULL);
   if ((end_index = strstr(line, end_text)) == NULL)
      return (NULL);

   /* adjustment cause strstr returns the start of the text */
   start_index += strlen(start_text);

   /* find length of text segment */
   segment_length = end_index - start_index;
   return g_strndup(start_index, segment_length);
}
