#ifndef __DEBUG_H__
#define __DEBUG_H__

/*
 * Simple debug messages. Add:
 *
 *    #define DEBUG_LEVEL <n>
 *    #include "debug.h"
 *
 * to the file you are working on, or let DEBUG_LEVEL undefined to
 * disable all messages. A higher level denotes a greater importance
 * of the message.
 */

#include <glib.h>

#ifdef __GNUC__

# ifdef DEBUG_LEVEL
#    define DEBUG_MSG(level, fmt...) \
        ( (DEBUG_LEVEL) && ((level) >= DEBUG_LEVEL) ) ? \
        g_print(fmt) : (level)
# else
#    define DEBUG_MSG(level, fmt...)
# endif /* DEBUG_LEVEL */


#define DEBUG_HTML_WARN 1

# ifdef DEBUG_HTML_WARN
#    define DEBUG_HTML_MSG(fmt...) g_print("HTML warning: " fmt)
# else
#    define DEBUG_HTML_MSG(fmt...)
# endif /* DEBUG_HTML_WARN */


#define DEBUG_HTTP_MSG(fmt...) g_print("HTTP warning: " fmt)

#else

#ifdef	DEBUG_LEVEL

#include	<stdarg.h>
static inline void debug_msg(int level,const char *fmt,...)
{
	if ( (DEBUG_LEVEL) && ((level) >= DEBUG_LEVEL) ) {
		va_list args;
		va_start(args,fmt);
		vprintf(fmt,args);
		va_end(args);
	}
}

#else
static inline void debug_msg(int level,const char *fmt,...) {}
#endif

#define	DEBUG_MSG	debug_msg
#define	DEBUG_HTML_MSG	g_print
#define	DEBUG_HTTP_MSG	g_print

#endif

#endif /* __DEBUG_H__ */


